#version 330

in vec3 worldPosition;
in vec2 texcoord;
in vec3 normal;

//// SHADOW MAPPING ////
in vec4 ShadowCoord;

out vec4 outColor;


// --------------------------
// GEOMETRY EFFECT PARAMETERS

uniform vec3 CameraPosition;
uniform vec3 AmbientLight;
uniform vec3 DiffuseLight;
uniform vec3 LightVector;
uniform bool FogEnabled;
uniform vec3 FogColor;
uniform float FogStart;
uniform float FogEnd;
uniform float WaterLevel;
uniform bool WaterClipping;


// -----------------------
// BASIC EFFECT PARAMETERS

uniform vec3 DiffuseColor;
uniform sampler2D DiffuseMap;
uniform bool DiffuseMapEnabled;
uniform vec3 EmissiveColor;
uniform sampler2D EmissiveMap;
uniform bool EmissiveMapEnabled;

// --------------------------
// SHADOW MAPPING	
uniform sampler2DShadow ShadowMap;


void main()
{
	vec3 color;

	// The current fragment is beneath the water level and water clipping is enabled.
	if (WaterClipping && WaterLevel > worldPosition.z)
	{
		// Throw away the whole fragment.
		discard;
	}

	// Calculate the distance between the fragment and the camera.
	vec3 viewVector = worldPosition - CameraPosition;
	float viewDistance = length(viewVector);

	// Use a solid diffuse color or a texture, if enabled.
	vec3 albedoColor = DiffuseColor;
	if (DiffuseMapEnabled)
	{
		albedoColor = texture(DiffuseMap, texcoord).rgb;
	}

	// Fetch the emissive color.
	vec3 emissiveLighting = EmissiveColor;
	if (EmissiveMapEnabled)
	{
		emissiveLighting = texture(EmissiveMap, texcoord).rgb;
	}

	// The light influence needs to be clamped to zero. Just because a triangle
	// faces away from the light source does not mean it should receive negative
	// lighting.
	float lightInfluence = clamp(dot(normal, LightVector), 0, 1);

	//// SHADOW MAPPING
	float cosTheta = dot(normal, LightVector);
	float bias = 0.0005*tan(acos(cosTheta)); 
	bias = clamp(bias, 0,0.002);
	float visibility = texture( ShadowMap, vec3(ShadowCoord.xy, (ShadowCoord.z / ShadowCoord.w) - bias));

	// Combine the lighting.
	vec3 ambientLighting = albedoColor * AmbientLight;
	vec3 diffuseLighting = visibility * albedoColor * DiffuseLight * lightInfluence;
	color = ambientLighting + diffuseLighting + emissiveLighting;

	if (FogEnabled)
	{
		// Add fog to the output color.
		float fogFactor =  clamp((FogEnd - viewDistance) / (FogEnd - FogStart), 0, 1);
		color = fogFactor * color + (1.0 - fogFactor) * FogColor;
	}

	outColor = vec4(color, 1);
}